// mpicc Ex3.c -Wall -o Ex3.x

#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

#include "par_child.h"

int myMPI_Bcast( void *buffer, int count, MPI_Datatype datatype,
                 int root, MPI_Comm comm )
{
    int myrank, nprocs;
    int tag = 0;

    int parent, left_ch, right_ch;

    MPI_Comm_rank( comm, &myrank );
    MPI_Comm_size( comm, &nprocs );

    get_parent_and_children( myrank, root, nprocs,
                             &parent, &left_ch, &right_ch );

    if ( parent != -1 )
    {
        // If I have a parent (i.e., I am not the root), receive from my parent
        MPI_Recv( buffer, count, datatype, parent, tag, comm, MPI_STATUS_IGNORE );
    }
    if ( left_ch != -1 )
    {
        // If I have a left child pass the message along
        MPI_Send( buffer, count, datatype, left_ch, tag, comm );
    }
    if ( right_ch != -1 )
    {
        // If I have a right child pass the message along
        MPI_Send( buffer, count, datatype, right_ch, tag, comm );
    }

    return 0;
}

int main( int argc, char *argv[] )
{   
    int i;
    int ROOT = 3; // Make sure to use -np p  with p > ROOT
    int myrank;
    int buffer[3];

    MPI_Init(&argc,&argv);

    MPI_Comm_rank( MPI_COMM_WORLD, &myrank );

    buffer[ 0 ] = -1;
    buffer[ 1 ] = -1;
    buffer[ 2 ] = -1;

    if ( myrank == ROOT )
    {
        buffer[ 0 ] = rand() % 20;
        buffer[ 1 ] = rand() % 20;
        buffer[ 2 ] = rand() % 20;
    }
    
    myMPI_Bcast( (void *)buffer, 3, MPI_INT, ROOT, MPI_COMM_WORLD );

    printf( "[%2d] %d %d %d\n", myrank, buffer[0], buffer[1], buffer[2] );

    MPI_Finalize();
    return 0;
}


