// mpicc Ex3.c -Wall -o Ex3.x

#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

#define INTS_PER_PROCESS 4

void create_file( char *fpath, int nints )
{
    int i, *buf;
    FILE *fp;

    buf = (int *) malloc ( nints * sizeof(int) );
    for ( i = 0; i < nints; i++ )
        buf[i] = i;

    fp = fopen( fpath, "wb" );
    fwrite( buf, sizeof(int), nints, fp );
    fclose( fp );

    free( buf );
}

int main( int argc, char *argv[] )
{   
    char *fpath;
    int root;
    int myrank, nprocs;

    int *file_buffer;
    int scatter_buf[ INTS_PER_PROCESS ];
    int reduce_buf[ INTS_PER_PROCESS ];

    MPI_Init(&argc,&argv);

    if ( argc != 3 )
    {
        fprintf( stderr, "Usage: %s file_path root\n", argv[0] );
        exit( -1 );
    }

    fpath = argv[1];
    root = atoi( argv[2] );

    MPI_Comm_rank( MPI_COMM_WORLD, &myrank );
    MPI_Comm_size( MPI_COMM_WORLD, &nprocs );

    // Root reads file
    if ( myrank == root )
    {
        FILE *fp;
        file_buffer = (int *) malloc ( nprocs * INTS_PER_PROCESS * sizeof(int) );

        // Create first (could create a large one in a separate program)
        create_file( fpath, nprocs * INTS_PER_PROCESS );

        fp = fopen( fpath, "rb" );
        fread( file_buffer, sizeof(int), INTS_PER_PROCESS * nprocs, fp );
        fclose( fp );
    }

    MPI_Scatter( file_buffer, INTS_PER_PROCESS, MPI_INT,
                 scatter_buf, INTS_PER_PROCESS, MPI_INT,
                 root, MPI_COMM_WORLD );

    MPI_Allreduce( scatter_buf, reduce_buf, INTS_PER_PROCESS, 
                   MPI_INT, MPI_SUM, MPI_COMM_WORLD );

    printf("[%2d] [ %2d, %2d, %2d, %2d ]\n", myrank, reduce_buf[0], reduce_buf[1],
                                                     reduce_buf[2], reduce_buf[3] );

    if ( myrank == root )
        free( file_buffer );

    MPI_Finalize();

    return 0;
}
